# -*- coding: utf-8 -*-

"""Main module."""

import hashlib
import logging
import ssl
import socket
import sys
import time

from . import colorlog


class Certo():
    """
    Main class. It expect two parameters to configure the logger:

    :param debugging: toggle the output of debugging messages.
    :param colored: toggle the colored log format
    """
    def __init__(self, debugging, colored):
        self.debugging = debugging
        self.colored = colored
        if debugging:
            self.log_level = logging.DEBUG
        else:
            self.log_level = logging.INFO

        if colored:
            self.formatter = colorlog.ColorFormatter("%(message)s")
        else:
            self.formatter = logging.Formatter("[%(levelname)4s] %(message)s")

        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(self.log_level)
        self._console_handler = logging.StreamHandler(sys.stdout)
        self._console_handler.setLevel(self.log_level)
        self._console_handler.setFormatter(self.formatter)
        self.logger.addHandler(self._console_handler)
        self.logger.debug("Formatter %r" % self.formatter)

    # The following inspired by:
    # https://stackoverflow.com/questions/17667903/python-socket-receive-large-amount-of-data
    # https://www.binarytides.com/receive-full-data-with-the-recv-socket-function-in-python/
    def recv_msg(self, sock, timeout):
        # Read message length and unpack it into an integer
        data = b''
        begin = time.time()
        n = 1024
        while len(data) < n or time.time() - begin < timeout:
            try:
                data += sock.recv(n - len(data))
                self.logger.debug("Partial: %r" % data)
            except socket.timeout:
                pass
            if b'Ready to start TLS\r\n' in data:
                break
            time.sleep(0.1)
        if len(data) == 0:
            return None
        self.logger.debug("Data: %r" % data)
        return data

    def establish_conn(self, addr, port, starttls):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(5)
        try:
            if starttls:
                self.logger.debug("Using STARTTLS")
                self.logger.debug("Connecting to %s:%s" % (addr, port))
                sock.connect((addr, port))
                sock.send(b"STARTTLS\r\n")
                data = self.recv_msg(sock, 5)
                if data is None:
                    raise socket.error
                wrapped_socket = ssl.wrap_socket(sock)
            else:
                wrapped_socket = ssl.wrap_socket(sock)
                wrapped_socket.connect((addr, port))

            return wrapped_socket.getpeercert(True)

        except socket.timeout:
            self.logger.error("Timeout trying to connect to %s:%s" % (addr, port))
            sys.exit(2)

        except socket.gaierror:
            self.logger.error("Failure in name resolution: %s" % addr)
            sys.exit(3)

        wrapped_socket.close()

    def get_cert(self, addr, port, starttls):
        cert = self.establish_conn(addr, port, starttls)
        pem_cert = ssl.DER_cert_to_PEM_cert(cert)
        self.logger.debug("The certificate is:\n%s" % pem_cert)

        return cert

    def capitalize_and_colons(self, in_hash):
        in_hash = in_hash.upper()
        new_hash = in_hash[0:2]
        for i in range(2, len(in_hash), 2):
            new_hash += ":" + in_hash[i:i+2]
        return new_hash

    def compute_fingerprints(self, cert, with_colons):
        thumb_md5 = hashlib.md5(cert).hexdigest()
        thumb_sha1 = hashlib.sha1(cert).hexdigest()
        thumb_sha256 = hashlib.sha256(cert).hexdigest()
        self.logger.info("MD5:    " + thumb_md5)
        if with_colons:
            self.logger.info("        " + self.capitalize_and_colons(thumb_md5))
        self.logger.info("SHA1:   " + thumb_sha1)
        if with_colons:
            self.logger.info("        " + self.capitalize_and_colons(thumb_sha1))
        self.logger.info("SHA256: " + thumb_sha256)
        if with_colons:
            self.logger.info("        " + self.capitalize_and_colons(thumb_sha256))
